﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3
#pragma version=1.00
#pragma IndependentModule=WiREloader
#include <Resize Controls>

// A prototype loader for Renishaw Raman files of type wdf.
// https://www.wavemetrics.com/user/tony
// 15/7/18

// Map files (including line scans) are loaded as multiple 1D spectra 
// with a common wave containing wavenumbers. Coordinates are recorded 
// in wave notes.

// updater headers
static constant kProjectID=21301 // the project node on IgorExchange
static strconstant ksShortTitle="WireLoader" // the project short title on IgorExchange

constant kMakeMapPlot=1

menu "Load Waves"
	"Load WiRE...", /Q, loadWiRE()
end

menu "Data"
	submenu "Packages"
		"WiRE map browser", /Q, makeMapBrowser()
	end
end

function loadWiRE()
	
	variable i
	open /D/R/MULT=1/F="WiRE Files (*.wdf):.wdf;"/M="Looking for WiRE files..." refnum
	wave /T w_selection=ListToTextWave(S_fileName, "\r")
	for (i=0;i<numpnts(w_selection);i+=1)
		loadWDF(w_selection[i])
	endfor
	return 1
end

function loadWDF(strPath)
	string strPath
		
	string strBlockName="", strData=""
	variable refnum=0, BlockLength=0, blockStart=0, i=0
	variable isMap=0, haveImageCoords=0, haveMapCoords=0
	variable ORGNstart=0, ORGNend=0, WXDMstart=0, WXDMend=0, WMAPstart=0
	variable testing=0
		
	make /free/n=0/D w_x, w_y
	make /free/n=0 w_wavenumber, w_data
	make /free/n=6 MapInfo
	make /free/B/n=0 w_jpeg // wave to hold any jpeg binary embedded in wdf
	make /free/n=4/d ImageCoords	
	wave /Z img=$""
	
	open /R/Z refnum as strPath
	fstatus refnum // sets V_logEOF, S_fileName
	
	if (v_flag==0)
		printf "WiRE loader: couldn't open %s\r", strPath
		return 0
	endif	
	printf "WiRE loader reading from %s%s\r", S_path, S_fileName
	
	string strBaseName=removeending(s_fileName, ".wdf")
	strBaseName=CleanupName(strBaseName, 0)
	strBaseName=removeending(strBaseName, "_")
	
	do // loop through blocks in file
		BlockLength=0
		strBlockName=""
		strBlockName=PadString(strBlockName, 4, 0 )
		
		FBinRead refnum, strBlockName
		if (i==0 && cmpstr(strBlockName, "WDF1"))
			printf "WiRE loader: %s does not appear to be a wdf file\r", s_fileName
			close refnum
			return 0
		endif
		FSetPos refnum, blockStart+8
		FBinRead /B=3/F=3/U refnum, BlockLength
		
		if(BlockLength==0)
			break
		endif
		
		strswitch(strBlockName)
			case "XLST":
				redimension /S/N=((BlockLength-24)/4) w_wavenumber
				FSetPos refnum, blockStart+24
				FBinRead /B=3 refnum, w_wavenumber
				break
			case "DATA":
				// if file contains map data this will be a concatenation of spectra
				redimension /S/N=((BlockLength-16)/4) w_data
				FSetPos refnum, blockStart+16
				FBinRead /B=3 refnum, w_data	
				break
			case "WMAP":
				isMap=1
				WMAPstart=blockStart
				FSetPos refnum, blockStart+24
				FBinRead /B=3/F=4 refnum, MapInfo // 6 points, mapX0, mapY0, unknown1, mapXstep, mapYstep, unknown2, 32 bit floats in wdf
				make /free/n=2 w_temp
				FBinRead /B=3/F=3/U refnum, w_temp 
				concatenate /NP {w_temp}, MapInfo // 7th and 8th points are mapWidth, mapHeight, 32 bit integers in wdf	
				break
			case "ORGN":
				ORGNstart=blockStart
				ORGNend=blockstart+blockLength
				break
			case "WHTL":
				redimension /N=((BlockLength-16)) w_jpeg
				FSetPos refnum, blockStart+16
				FBinRead /B=3 refnum, w_jpeg	 // read jpeg binary into w_jpeg
				break
			case "WXDM": // here is where we find image registration
				WXDMstart=blockstart
				WXDMend=blockstart+blockLength
				break
		endswitch
		
		if (testing)
			printf "%s start: %.10g length: %.10g bytes\r", strBlockName, blockStart, blockLength
		endif
		
		blockStart=blockStart+BlockLength
		
		if (blockStart>=V_logEOF)
			break
		endif
		
		FSetPos refnum, blockStart
		
		i+=1		
	while(i<1000)
	
	if (isMap && ORGNstart)
		haveMapCoords=readXYpositions(w_x, w_y, ORGNstart, ORGNend, refnum)	
	endif
	
	if(numpnts(w_jpeg) && WXDMstart)
		haveImageCoords=readImageCoords(ImageCoords, WXDMstart, WXDMend, refnum)
	endif

	close refnum // done with reading data from file
		
	if (numpnts(w_data)==0)
		printf "WiRE loader error: could not locate data block in %s\r", S_fileName
		return 0
	endif
	
	if (numpnts(w_wavenumber)==0)
		printf "WiRE loader error: could not locate wavenumber block in %s\r", S_fileName
		return 0
	endif
	
	variable Xlength=numpnts(w_wavenumber)
	variable dataLength=numpnts(w_data)
	if (mod(datalength, xlength) != 0)
		print "WiRE loader errror: mismatched wavelengths in "+S_fileName
		return 0
	endif
	variable numSpectra=dataLength/Xlength
			
	string xName=strBaseName+"_x"
	
	// check for waves to be overwritten
	string overwriteList=selectstring(exists(xName), "", xName+";")

	if((numpnts(w_jpeg)>0))
		overwriteList+=selectstring(exists(strBaseName+"_img"), "", strBaseName+"_img;")
	endif
	
	if(haveMapCoords)
		overwriteList+=selectstring(exists(strBaseName+"_mapX"), "", strBaseName+"_mapX;")
		overwriteList+=selectstring(exists(strBaseName+"_mapY"), "", strBaseName+"_mapY;")
	endif
	
	if(isMap)
		for(i=0;i<numSpectra;i+=1)
			overwriteList+=selectstring(exists(strBaseName+"_"+num2str(i)), "", strBaseName+"_"+num2str(i)+";")
		endfor
	else
		overwriteList+=selectstring(exists(strBaseName), "", strBaseName+";")
	endif
	
	if (ItemsInList(overwriteList))
		DoAlert 1, DoAlertString(OverwriteList)
		if(v_flag==2)
			print s_fileName+" load aborted"
			return 0
		endif
	endif
	
	// save Raman shift wave
	duplicate /O w_wavenumber $xName
	printf "Loaded Raman shifts as %s\r", xName
	
	if (!isMap) // for single point save intensity wave and we're done
		note w_data "Xwave:"+xName
		duplicate /O w_data $strBaseName
		printf "Loaded intensity as %s\r", strBaseName
		return 1
	endif
	
	printf "Map file with %g spectra, ", numSpectra
	printf "origin=(%g,%g), step=(%g,%g), size=(%d,%d)\r", MapInfo[0], MapInfo[1], MapInfo[3], MapInfo[4], MapInfo[6], MapInfo[7]
	printf "Loaded intensity spectra as"
	
	string cmd=""
	i=0
	do
		make /o/n=(xlength) $(strBaseName+"_"+num2str(i)) /wave=ithDataWave
		ithDataWave = w_data[p+i*xlength]
		cmd=""
		if (haveMapCoords)
			sprintf cmd "Coordinates: X=%f, Y=%f\r", w_x[i], w_y[i]
		endif
		sprintf cmd, "%sXwave: %s", cmd, xName
		note ithDataWave cmd
		printf selectstring(mod(i,10)==0, "", "\r")
		printf " %s", nameofwave(ithDataWave) // record wavenames in history
		i+=1
	while( (i*xlength) < datalength)
	printf "\r"
		
	// save image
	if(numpnts(w_jpeg))
		wave /Z img=saveImage(w_jpeg, strBaseName)
	endif
	
	// same map coordinates
	if(haveMapCoords)	
		duplicate /o w_x $(strbasename+"_mapX") /wave=wMapX
		duplicate /o w_y $(strbasename+"_mapY") /wave=wMapY
		printf "Loaded map coordinates as %s_mapX, %s_mapY\r", strbasename, strbasename
	endif
	
	if(waveexists(img) && haveImageCoords)
		setscale /I x, ImageCoords[0], ImageCoords[1], img
		setscale /I y, ImageCoords[2], ImageCoords[3], img	
		if(kMakeMapPlot) // plot image and overlay map points
			if (haveMapCoords)
				makeMapBrowser(strSelect=strBaseName)			
			else
				Display /K=1;AppendImage img
				ModifyGraph standoff=0, height={Plan,1,left,bottom}
				SetAxis/A/R left
			endif
		endif
	endif

	return 1	
end

function readImageCoords(ImageCoords, WXDMstart, WXDMend, refnum)
	wave  ImageCoords
	variable WXDMstart, WXDMend, refnum
	
	make /B/U/free/n=8 w8bytes, w_match
	w_match={77,97,112,65,114,101,97,98} // "MapAreab" ASCII
	variable pos=WXDMstart+8
	FSetPos refnum, WXDMstart+8
	string oneByte=" "
	for(pos=WXDMstart+8;pos<=WXDMend;pos+=1)
		FBinRead /B=3 refnum, oneByte
		if(stringmatch(oneByte, "M"))
			FsetPos refnum, pos
			FBinRead /B=3 refnum, w8bytes
			w8bytes-=w_match
			if(sum(w8bytes)==0)
				break
			endif
			FsetPos refnum, pos+1
		endif
	endfor
	if(pos<WXDMend)
		FsetPos refnum, pos+192
		FBinRead /B=3/F=5 refnum, ImageCoords
		return 1
	endif
	return 0
end

function readXYpositions(w_x, w_y, blockStart, blockEnd, refnum)
	wave w_x, w_y
	variable blockStart, blockEnd, refnum
	
	variable XstartByte, YstartByte
	variable Xcount=0, Ycount=0
	make /B/U/free/n=8 w8bytes, w_match
	
	w_match={4,0,0,128,5,0,0,0} // the 8 bytes at the end of X values
	XstartByte=blockStart+11*4
	FSetPos refnum, XstartByte
	variable countMax=(blockEnd-XstartByte)/8
	do
		FBinRead /B=3 refnum, w8bytes
		if(w8bytes[0]==4)
			w8bytes-=w_match
			if(sum(w8bytes)==0)
				break
			endif
		endif
		Xcount+=1	
	while (Xcount<countMax)
	
	if(Xcount==countMax)
		return 0
	endif
	
	FGetPos refNum
	
	YstartByte=V_filePos+16 // skip 16 bytes
	FSetPos refnum, YstartByte
	
	w_match={11,0,0,0,24,0,0,0} // the 8 bytes at the end of Y values
	countMax=(blockEnd-YstartByte)/8+1 // beyond end of block
	do
		FBinRead /B=3 refnum, w8bytes
		if(w8bytes[0]==11)
			w8bytes-=w_match
			if(sum(w8bytes)==0)
				break
			endif
		endif
		Ycount+=1	
	while (Ycount<countMax)
	
	if(Ycount==countMax)
		return 0
	endif
	
	if(Xcount!=Ycount)
		return 0
	endif
	
	// coordinates in WDF file have double precision	
	Redimension /D/N=(Xcount) w_x
	Redimension /D/N=(Ycount) w_y
	
	FSetPos refnum, XstartByte
	FBinRead /B=3 refnum, w_x
	FSetPos refnum, YstartByte
	FBinRead /B=3 refnum, w_y
	
	return 1	
end

// turns raw jpeg binary in w_jpeg into an image wave 
function /WAVE saveImage(w_jpeg, strBaseName)
	wave /B w_jpeg
	string strBaseName
	
	variable refnum
	string tempPath=SpecialDirPath("Temporary", 0, 0, 0)
	open /Z refnum as tempPath+"WiREJPEG.jpg"
	FBinWrite refnum, w_jpeg
	close refnum
	ImageLoad/Q/Z/T=jpeg/N=$(strBaseName+"_img")/O tempPath+"WiREJPEG.jpg"
	if(V_flag)
		wave img=$stringfromlist(0, S_waveNames)
		printf "Loaded %d x %d image as %s\r", dimsize(img, 0), dimsize(img, 1), nameofwave(img)
		return img
	endif
	return $""
end

function /S DoAlertString(OverwriteList)
	string OverwriteList
	
	string cmd="The following waves will be overwritten:\r"
	string tooMany= "\r*** TOO MANY WAVES TO LIST ***\r"
	string cont="\rContinue?"
	variable cmdLen = strlen(cmd)+strlen(tooMany)+strlen(cont)
	variable i
	variable numWaves=itemsinlist(OverwriteList)
	string ithWave
	for (i=0;i<numWaves;i+=1)
		ithWave=StringFromList(i, OverwriteList)
		// avoid too-long alert
		if( cmdLen+strlen(ithWave) >= 1023 )
			cmd+=tooMany
			break
		endif
		cmd+=ithWave+", "
		cmdLen+=strlen(ithWave+", ")
	endfor
	cmd+=cont
	return cmd
end

// --------------------- map browser functions -------------------

function makeMapBrowser([strSelect])
	string strSelect
	
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:MapBrowser
	DFREF dfr = root:Packages:MapBrowser
	make /o/n=1 dfr:pointY /wave=pointY, dfr:pointX /wave=pointX
	pointY=nan; pointX=nan
	variable /G dfr:selectedPoint=nan
	make /o/n=0/T dfr:toPlot
	make /o/n=0/WAVE dfr:allWaves
	make /o dfr:preview /wave=preview, dfr:previewX /wave=previewX
	preview=nan; previewX=nan
	
	killWindow /z MapBrowserPanel
	NewPanel /K=1/N=MapBrowserPanel/W=(50,50,860,390) as "Map Browser"
	
	DefineGuide/W=MapBrowserPanel guidePreviewRight={FR,-20}
	DefineGuide/W=MapBrowserPanel guideMapLeft={FL,20}
	DefineGuide/W=MapBrowserPanel guideCentre={FL,0.5,FR}
	DefineGuide/W=MapBrowserPanel guideMapRight={guideCentre, -10}
	DefineGuide/W=MapBrowserPanel guidePreviewLeft={guideCentre, 10}
	DefineGuide/W=MapBrowserPanel guideTop={FT,20}
	DefineGuide/W=MapBrowserPanel guideMapBottom={FB,-80}
	DefineGuide/W=MapBrowserPanel guidePreviewBottom={guideMapBottom,40}
		
	Display/HOST=MapBrowserPanel/FG=(guideMapLeft,guideTop,guideMapRight,guideMapBottom)/PG=(guideMapLeft,guideTop,guideMapRight,guideMapBottom)  /N=MapPlot
	AppendToGraph /W=MapBrowserPanel#MapPlot pointY vs pointX
	ModifyGraph /W=MapBrowserPanel#MapPlot marker(pointY)=1,rgb(pointY)=(0,65535,0)
	
	Display/HOST=MapBrowserPanel/FG=(guidePreviewLeft,guideTop,guidePreviewRight,guidePreviewBottom)/PG=(guidePreviewLeft,guideTop,guidePreviewRight,guideMapBottom) /N=PreviewPlot
	AppendToGraph /W=MapBrowserPanel#PreviewPlot preview vs previewX
	ModifyGraph /W=MapBrowserPanel#PreviewPlot rgb(preview)=(65535,0,0)
	ModifyGraph /W=MapBrowserPanel#PreviewPlot frameStyle=0, noLabel(left)=2, wbRGB=(61166,61166,61166)

	variable vMode=1
	if(paramisdefault(strSelect)==0)
		vMode=1+WhichListItem(strSelect+"_img", wavelist("*_img",";","DIMS:3"))
		vMode = (vMode==0) ? 1 : vMode
	endif
	PopupMenu popupImage,win=MapBrowserPanel,pos={20,270},size={124,23}, mode=vMode, title="Map", fsize=14
	PopupMenu popupImage,win=MapBrowserPanel,value=wavelist("*_img",";","DIMS:3"), proc=MapBrowserPopupProc	
	Button buttonSelectAll,win=MapBrowserPanel,pos={20,300},size={70,25},title="Select All",proc=MapBrowserButtonProc
	Button buttonClear,win=MapBrowserPanel,pos={110,300},size={45,25},title="Clear",proc=MapBrowserButtonProc	
	Button buttonNewGraph,win=MapBrowserPanel,pos={792-80,300},size={80,25},title="New Graph"
	Button buttonNewGraph,win=MapBrowserPanel,disable=2,proc=MapBrowserButtonProc
	Button buttonAppendToTop,win=MapBrowserPanel,pos={792-80-20-100,300},size={100,25},title="Append to Top"
	Button buttonAppendToTop,win=MapBrowserPanel,disable=2, proc=MapBrowserButtonProc
	
	SetWindow MapBrowserPanel, activeChildFrame=0	
	SetWindow MapBrowserPanel, hook(MBhook)=WireLoader#MapBrowserHook	
	SetActiveSubwindow MapBrowserPanel
	
	// resizing user data for controls
	PopupMenu popupImage,win=MapBrowserPanel,userdata(ResizeControlsInfo)= A"!!,BY!!#BA!!#>Z!!#<Xz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	PopupMenu popupImage,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	PopupMenu popupImage,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonSelectAll,win=MapBrowserPanel,userdata(ResizeControlsInfo)= A"!!,BY!!#BP!!#?E!!#=+z!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	Button buttonSelectAll,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonSelectAll,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonClear,userdata(ResizeControlsInfo)= A"!!,FA!!#BP!!#>B!!#=+z!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	Button buttonClear,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonClear,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonNewGraph,win=MapBrowserPanel,userdata(ResizeControlsInfo)= A"!!,JC!!#BP!!#?Y!!#=+z!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz"
	Button buttonNewGraph,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonNewGraph,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonAppendToTop,win=MapBrowserPanel,userdata(ResizeControlsInfo)= A"!!,J%!!#BP!!#@,!!#=+z!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz"
	Button buttonAppendToTop,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonAppendToTop,win=MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	
	// resizing user data for panel window
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfo)= A"!!*'\"z!!#DZJ,hs:zzzzzzzzzzzzzzzzzzzzz"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzzzzzzzz"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzz!!!"
	
	SetWindow MapBrowserPanel,userdata(ResizeControlsGuides)=  "guidePreviewRight;guideMapLeft;guideCentre;guideMapRight;guidePreviewLeft;guideTop;guideMapBottom;guidePreviewBottom;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguidePreviewRight)=  "NAME:guidePreviewRight;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:790.00;GUIDE1:FR;GUIDE2:;RELPOSITION:-20;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguideMapLeft)=  "NAME:guideMapLeft;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:20.00;GUIDE1:FL;GUIDE2:;RELPOSITION:20;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguideCentre)=  "NAME:guideCentre;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:405.00;GUIDE1:FL;GUIDE2:FR;RELPOSITION:0.5;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguideMapRight)=  "NAME:guideMapRight;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:395.00;GUIDE1:guideCentre;GUIDE2:;RELPOSITION:-10;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguidePreviewLeft)=  "NAME:guidePreviewLeft;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:0;POSITION:415.00;GUIDE1:guideCentre;GUIDE2:;RELPOSITION:10;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguideTop)=  "NAME:guideTop;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:1;POSITION:20.00;GUIDE1:FT;GUIDE2:;RELPOSITION:20;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguideMapBottom)=  "NAME:guideMapBottom;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:1;POSITION:260.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-80;"
	SetWindow MapBrowserPanel,userdata(ResizeControlsInfoguidePreviewBottom)=  "NAME:guidePreviewBottom;WIN:MapBrowserPanel;TYPE:User;HORIZONTAL:1;POSITION:300.00;GUIDE1:guideMapBottom;GUIDE2:;RELPOSITION:40;"
		
	// resizing panel hook
	SetWindow MapBrowserPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook

	// use popup proc to select an image
	ControlInfo /W=MapBrowserPanel popupImage
	struct WMPopupAction s
	s.eventCode=2
	s.popStr=S_Value
	MapBrowserPopupProc(s)
end

function MapBrowserPopupProc(s)
	STRUCT WMPopupAction &s
	
	if(s.eventCode!=2)
		return 0
	elseif(strlen(s.popStr)==0 || stringmatch(s.popStr, "_none_"))
		return 0
	endif
	
	string basename=removeending(s.popStr, "img")
	string oldImg=stringfromlist(0, ImageNameList("MapBrowserPanel#MapPlot", ";"))
	
	wave NewImg=$s.popStr
	wave NewMap=$basename+"mapY"
	wave NewMapX=$basename+"mapX"
	wave data=$basename+"0"
	wave dataX=$basename+"x"
	
	variable differentMap=(stringmatch(oldImg, s.popStr)==0)
	
	do
		RemoveImage /W=MapBrowserPanel#MapPlot/Z  $stringfromlist(0, ImageNameList("MapBrowserPanel#MapPlot", ";"))
	while(strlen(ImageNameList("MapBrowserPanel#MapPlot", ";")))
	do
		RemoveFromGraph /W=MapBrowserPanel#MapPlot/Z  $stringfromlist(0, TraceNameList("MapBrowserPanel#MapPlot", ";", 1))
	while(strlen(TraceNameList("MapBrowserPanel#MapPlot", ";", 1)))

	AppendImage /W=MapBrowserPanel#MapPlot NewImg
	AppendToGraph /W=MapBrowserPanel#MapPlot NewMap /TN=map vs NewMapX 
	ModifyGraph /W=MapBrowserPanel#MapPlot rgb=(65535,0,0), highTrip=100000
	SetAxis/A/R /W=MapBrowserPanel#MapPlot left
	SetAxis/A /W=MapBrowserPanel#MapPlot bottom
		
	DFREF dfr = root:Packages:MapBrowser
	
	wave /Z pointY=dfr:pointY
	wave /Z pointX=dfr:pointX
	wave /Z/T toPlot=dfr:toPlot

	AppendToGraph /W=MapBrowserPanel#MapPlot pointY vs pointX
	ModifyGraph /W=MapBrowserPanel#MapPlot mode=3,marker=0
	ModifyGraph /W=MapBrowserPanel#MapPlot rgb(pointY)=(0,65535,0)
		
	duplicate /o data dfr:preview /wave=preview
	duplicate /o dataX dfr:previewX /wave=previewX
	preview=nan
	SetAxis /A/W=MapBrowserPanel#PreviewPlot
	ModifyGraph /W=MapBrowserPanel#PreviewPlot noLabel(left)=2 
	
	wave /wave allwaves=dfr:allWaves
	redimension /N=(numpnts(NewMap)) allWaves
	allWaves=$(getDataFolder(1)+basename+num2str(p))
	
	if(differentMap)		
		do // clear preview plot
			RemoveFromGraph /W=MapBrowserPanel#PreviewPlot/Z  $stringfromlist(0, TraceNameList("MapBrowserPanel#PreviewPlot", ";", 1))
		while(strlen(TraceNameList("MapBrowserPanel#PreviewPlot", ";", 1)))
		AppendToGraph /W=MapBrowserPanel#PreviewPlot /C=(65535,0,0) preview vs previewX
		// clear seleced points
		redimension /N=1 pointY, pointX
		redimension /N=0 toPlot
		pointX=nan; pointY=nan
	endif
	
	return 0
end

function MapBrowserButtonProc(s)
	STRUCT WMButtonAction &s
	
	variable hist=1 // choose whether to record graph building commands to history
	
	if(s.eventCode!=2)
		return 0
	endif
		
	strswitch(s.ctrlName)
		case "buttonSelectAll":
			SelectAll(1)
			return 0
		case "buttonClear":
			SelectAll(0)
			return 0
		case "buttonNewGraph":
			Display
			if (hist)
				print "•Display"
			endif
		case "buttonAppendToTop":
			// make sure we have a top graph
			string strTopGraph=WinName(0, 1)
			if(strlen(strTopGraph)==0)
				return 0
			endif
			dowindow /F $strTopGraph
			
			DFREF packageDfr = root:Packages:MapBrowser
			wave /T toPlot=packageDfr:toPlot
			
			variable i, imax=numpnts(toPlot)
			string cmd=""
			
			for(i=0;i<imax;i+=1)
				wave w=$toPlot[i]
				wave w_x=MapBrowserXWaveRef(w)
				appendtograph w vs w_x
				if (hist)
					printf "•AppendToGraph %s vs %s\r", nameofwave(w), nameofwave(w_x)
				endif
			endfor
			
			if(stringmatch(s.ctrlName, "buttonNewGraph"))
				Label/Z left "Intensity"
				Label/Z bottom "Wavenumber (cm\\S-1\\M)"
			endif
			break
	endswitch	
	return 0
end

function MapBrowserHook(s)
	STRUCT WMWinHookStruct &s	
	
	Variable hookResult = 0
	DFREF packageDfr=root:packages:MapBrowser
	NVAR selectedPoint=packageDfr:selectedPoint
		
	switch(s.eventCode)
		case 4: // mousemoved	
			wave /Z/SDFR=packageDfr pointY,pointX,preview
			if (numpnts(pointY)==0)
				break
			endif
			variable pnt=numpnts(pointY)-1
			string keyList=TraceFromPixel(s.mouseloc.h,s.mouseloc.v, "WINDOW:MapBrowserPanel#MapPlot;ONLY:map;")			
			if (strlen(keyList))
				wave /Z mapY=TraceNameToWaveRef("MapBrowserPanel#MapPlot", "map")
				wave /Z mapX=XWaveRefFromTrace("MapBrowserPanel#MapPlot", "map")
				DFREF dfr=GetWavesDataFolderDFR(TraceNameToWaveRef("MapBrowserPanel#MapPlot", "map"))
				string basename=RemoveEnding(nameofwave(mapY), "MapY")			
				string strHitP=StringByKey("HITPOINT", keyList)
				selectedPoint=str2num(strHitP)
				wave /Z/SDFR=dfr xwave=$basename+"x"
				wave /Z/SDFR=dfr datawave=$basename+strHitP
							
				if (waveexists(xwave)==0 || waveexists(datawave)==0)
					break
				endif
				
				pointY[pnt]=mapY[selectedPoint]
				pointX[pnt]=mapX[selectedPoint]
				preview=datawave
				reorderTraces /W=MapBrowserPanel#PreviewPlot _front_, {preview}
				ModifyGraph /W=MapBrowserPanel#PreviewPlot noLabel(left)=2 
			else // not on point
				pointY[pnt]=NaN; pointX[pnt]=NaN
				selectedPoint=nan
				preview=nan
			endif
			break
		case 5:	// mouseup
			if(numtype(selectedPoint)==2) // nan
				return hookResult
			endif
			selectPoint(selectedPoint, -1)
			break
	endswitch
	
	return hookResult	// If non-zero, we handled event and Igor will ignore it.
end

// select/deselect a point from the map by number, add selected spectrum to preview plot
function selectPoint(pnt, select)
	variable pnt, select // select = 1 to select, 0 to deselect, -1 to toggle selection
	
	DFREF dfr=root:packages:MapBrowser
	wave /Z/SDFR=dfr pointX, pointY
	wave /Z/T/SDFR=dfr toPlot
	wave /Z/Wave/SDFR=dfr allWaves
	wave /Z mapY=TraceNameToWaveRef("MapBrowserPanel#MapPlot", "map")
	wave /Z mapX=XWaveRefFromTrace("MapBrowserPanel#MapPlot", "map")
	string waveName=nameofwave(allWaves[pnt])
		
	findValue /TEXT=waveName/TXOP=4 toPlot
	if(select==-1) // toggle selection
		select=!(	V_Value>-1)
	endif
	if(select == (V_Value>-1)) // already selected/deselected
		return 0	
	endif
	
	if (select==0)
		deletepoints V_Value, 1, toPlot, pointX, pointY
		removeFromGraph /W=MapBrowserPanel#PreviewPlot $waveName
		wave preview=dfr:preview
		preview=nan
	else
		wave w=allWaves[pnt]
		toPlot[numpnts(toPlot)]={nameofwave(w)}
		pointX[numpnts(pointX)-1]=mapX[pnt]
		pointY[numpnts(pointY)-1]=mapY[pnt]
		pointX[numpnts(pointX)]={nan}; pointY[numpnts(pointY)]={nan}
		AppendToGraph /C=(2,39321,1)/W=MapBrowserPanel#PreviewPlot w vs MapBrowserXWaveRef(w)
		ModifyGraph /W=MapBrowserPanel#PreviewPlot noLabel(left)=2
	endif
	
	Button buttonNewGraph,win=MapBrowserPanel, disable=2*(numpnts(toPlot)==0)
	Button buttonAppendToTop,win=MapBrowserPanel, disable=2*(numpnts(toPlot)==0)
	return 1
end

// returns waveref for the X wave paired with wave w
function /WAVE MapBrowserXWaveRef(w)
	wave w
	
	if(waveexists(w)==0)
		return $""
	endif
	DFREF dfr=GetWavesDataFolderDFR(w)
	string s_x=stringbyKey("Xwave", note(w), ":", "\r")
	if(cmpstr(s_x[0]," ")==0)
		do // remove whitespace after key separator
			s_x=ReplaceString(" ", s_x, "", 0, 1)  
		while((cmpstr(s_x[0]," ")==0)&&strlen(s_x))
	endif
	do
		s_x=removeEnding(s_x, " ")
	while(cmpstr(s_x[strlen(s_x)-1]," ")==0)
	
	return dfr:$s_x
end

Menu "GraphMarquee", dynamic
	
	MenuString(1), /Q, SelectPointsInMarquee(1)
	MenuString(0), /Q, SelectPointsInMarquee(0)
end

function /s MenuString(select)
	variable select
	
	if(stringmatch(WinList("*", ";", "WIN:"), "MapBrowserPanel;"))
		return selectstring(select, "Deselect", "Select") + " points in marquee"
	endif
	return ""	
end

function SelectPointsInMarquee(select)
	variable select
		
	wave /Z mapY=TraceNameToWaveRef("MapBrowserPanel#MapPlot", "map")
	wave /Z mapX=XWaveRefFromTrace("MapBrowserPanel#MapPlot", "map")	
	make /free/n=0 xPoly, yPoly
	getmarquee /K/Z left, bottom
	xPoly={v_left,v_left,v_right,v_right}
	yPoly={v_top, v_bottom, v_bottom, v_top}	
	FindPointsInPoly mapX, mapY, xPoly, yPoly
	wave W_inPoly=W_inPoly
	W_inPoly = W_inPoly ? selectPoint(p,select) : 0
end

function SelectAll(select)
	variable select
		
	wave /Z mapY=TraceNameToWaveRef("MapBrowserPanel#MapPlot", "map")
	make /free /N=(numpnts(mapY)) w = selectPoint(p,select) // loop over points of map
end